"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sarus_data_spec.protobuf.path_pb2
import sarus_data_spec.protobuf.statistics_pb2
import sarus_data_spec.protobuf.type_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class Transform(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    class ExternalOp(_ExternalOp, metaclass=_ExternalOpEnumTypeWrapper):
        pass
    class _ExternalOp:
        V = typing.NewType('V', builtins.int)
    class _ExternalOpEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ExternalOp.V], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
        PD_MEAN = Transform.ExternalOp.V(0)
        PD_STD = Transform.ExternalOp.V(1)
        PD_LOC = Transform.ExternalOp.V(2)
        PD_EQ = Transform.ExternalOp.V(3)
        NP_MEAN = Transform.ExternalOp.V(4)
        NP_STD = Transform.ExternalOp.V(5)
        NP_ADD = Transform.ExternalOp.V(6)
        NP_SUB = Transform.ExternalOp.V(7)
        TF_BATCH = Transform.ExternalOp.V(8)
        TF_MAP = Transform.ExternalOp.V(9)
        SK_FIT = Transform.ExternalOp.V(10)
        SK_DP_FIT = Transform.ExternalOp.V(11)
        ADD = Transform.ExternalOp.V(12)
        SUB = Transform.ExternalOp.V(13)
        MUL = Transform.ExternalOp.V(14)
        DIV = Transform.ExternalOp.V(15)
        PD_ANY = Transform.ExternalOp.V(16)
        INVERT = Transform.ExternalOp.V(17)
        GETITEM = Transform.ExternalOp.V(18)
        SK_SCALE = Transform.ExternalOp.V(19)
        LEN = Transform.ExternalOp.V(20)
        PD_PROFILE_REPORT = Transform.ExternalOp.V(21)
        PD_DESCRIBE = Transform.ExternalOp.V(22)
        PD_SELECT_DTYPES = Transform.ExternalOp.V(23)
        PD_QUANTILE = Transform.ExternalOp.V(24)
        PD_SUM = Transform.ExternalOp.V(25)
        PD_FILLNA = Transform.ExternalOp.V(26)
        PD_ROUND = Transform.ExternalOp.V(27)
        PD_RENAME = Transform.ExternalOp.V(28)
        PD_COUNT = Transform.ExternalOp.V(29)
        PD_TRANSPOSE = Transform.ExternalOp.V(30)
        PD_UNIQUE = Transform.ExternalOp.V(31)
        """class method"""

        PD_VALUE_COUNTS = Transform.ExternalOp.V(32)
        PD_TO_DICT = Transform.ExternalOp.V(33)
        PD_APPLY = Transform.ExternalOp.V(34)
        PD_SUB = Transform.ExternalOp.V(35)
        PD_MEDIAN = Transform.ExternalOp.V(36)
        PD_ABS = Transform.ExternalOp.V(37)
        PD_MAD = Transform.ExternalOp.V(38)
        PD_SKEW = Transform.ExternalOp.V(39)
        PD_KURTOSIS = Transform.ExternalOp.V(40)
        PD_AGG = Transform.ExternalOp.V(41)
        PD_DROPLEVEL = Transform.ExternalOp.V(42)
        PD_SORT_VALUES = Transform.ExternalOp.V(43)
        PD_DROP = Transform.ExternalOp.V(44)
        PD_CORR = Transform.ExternalOp.V(45)
        GT = Transform.ExternalOp.V(46)
        GE = Transform.ExternalOp.V(47)
        LT = Transform.ExternalOp.V(48)
        LE = Transform.ExternalOp.V(49)
        NE = Transform.ExternalOp.V(50)
        POS = Transform.ExternalOp.V(51)
        NEG = Transform.ExternalOp.V(52)
        ABS = Transform.ExternalOp.V(53)
        ROUND = Transform.ExternalOp.V(54)
        MOD = Transform.ExternalOp.V(55)
        OR = Transform.ExternalOp.V(56)
        AND = Transform.ExternalOp.V(57)
        SK_ONEHOT = Transform.ExternalOp.V(99)
        SK_PCA = Transform.ExternalOp.V(100)
        SK_AFFINITY_PROPAGATION = Transform.ExternalOp.V(101)
        """Cluster"""

        SK_AGGLOMERATIVE_CLUSTERING = Transform.ExternalOp.V(102)
        SK_BIRCH = Transform.ExternalOp.V(103)
        SK_DBSCAN = Transform.ExternalOp.V(104)
        SK_FEATURE_AGGLOMERATION = Transform.ExternalOp.V(105)
        SK_KMEANS = Transform.ExternalOp.V(106)
        SK_MINIBATCH_KMEANS = Transform.ExternalOp.V(107)
        SK_MEAN_SHIFT = Transform.ExternalOp.V(108)
        SK_OPTICS = Transform.ExternalOp.V(109)
        SK_SPECTRAL_CLUSTERING = Transform.ExternalOp.V(110)
        SK_SPECTRAL_BICLUSTERING = Transform.ExternalOp.V(58)
        SK_SPECTRAL_COCLUSTERING = Transform.ExternalOp.V(59)
        SK_ADABOOST_CLASSIFIER = Transform.ExternalOp.V(60)
        """ensemble"""

        SK_ADABOOST_REGRESSOR = Transform.ExternalOp.V(61)
        SK_BAGGING_CLASSIFIER = Transform.ExternalOp.V(62)
        SK_BAGGING_REGRESSOR = Transform.ExternalOp.V(63)
        SK_EXTRA_TREES_REGRESSOR = Transform.ExternalOp.V(64)
        SK_EXTRA_TREES_CLASSIFIER = Transform.ExternalOp.V(65)
        SK_GRADIENT_BOOSTING_CLASSIFIER = Transform.ExternalOp.V(66)
        SK_GRADIENT_BOOSTING_REGRESSOR = Transform.ExternalOp.V(67)
        SK_ISOLATION_FOREST = Transform.ExternalOp.V(68)
        SK_RANDOM_FOREST_CLASSIFIER = Transform.ExternalOp.V(69)
        SK_RANDOM_FOREST_REGRESSOR = Transform.ExternalOp.V(70)
        SK_RANDOM_TREES_EMBEDDING = Transform.ExternalOp.V(71)
        SK_STACKING_CLASSIFIER = Transform.ExternalOp.V(72)
        SK_STACKING_REGRESSOR = Transform.ExternalOp.V(73)
        SK_VOTING_CLASSIFIER = Transform.ExternalOp.V(74)
        SK_VOTING_REGRESSOR = Transform.ExternalOp.V(75)
        SK_HIST_GRADIENT_BOOSTING_REGRESSOR = Transform.ExternalOp.V(76)
        SK_HIST_GRADIENT_BOOSTING_CLASSIFIER = Transform.ExternalOp.V(77)
        SK_CROSS_VAL_SCORE = Transform.ExternalOp.V(78)
        """Model selection"""

        SK_TRAIN_TEST_SPLIT = Transform.ExternalOp.V(79)
        SK_REPEATED_STRATIFIED_KFOLD = Transform.ExternalOp.V(80)
        SK_ACCURACY_SCORE = Transform.ExternalOp.V(81)
        """Metrics"""

        SK_AVERAGE_PRECISION_SCORE = Transform.ExternalOp.V(82)
        SK_CLASSIFICATION_REPORT = Transform.ExternalOp.V(83)
        SK_CONFUSION_MATRIX = Transform.ExternalOp.V(84)
        SK_F1_SCORE = Transform.ExternalOp.V(85)
        SK_PLOT_CONFUSION_MATRIX = Transform.ExternalOp.V(86)
        SK_PRECISION_RECALL_CURVE = Transform.ExternalOp.V(87)
        SK_PRECISION_SCORE = Transform.ExternalOp.V(88)
        SK_RECALL_SCORE = Transform.ExternalOp.V(89)
        SK_ROC_AUC_SCORE = Transform.ExternalOp.V(90)
        SK_ROC_CURVE = Transform.ExternalOp.V(91)
        XGB_CLASSIFIER = Transform.ExternalOp.V(92)
        """xgb"""

        XGB_CLASSIFIER_PREDICT = Transform.ExternalOp.V(93)
        PD_GET_DUMMIES = Transform.ExternalOp.V(94)
        """Pandas"""

        PD_JOIN = Transform.ExternalOp.V(95)
        PD_CONCAT = Transform.ExternalOp.V(96)
        SK_RANDOM_FOREST_CLASSIFIER_PREDICT = Transform.ExternalOp.V(97)
        """ensemble predict"""

        SK_LABEL_ENCODER_FIT_TRANSFORM = Transform.ExternalOp.V(98)

    PD_MEAN = Transform.ExternalOp.V(0)
    PD_STD = Transform.ExternalOp.V(1)
    PD_LOC = Transform.ExternalOp.V(2)
    PD_EQ = Transform.ExternalOp.V(3)
    NP_MEAN = Transform.ExternalOp.V(4)
    NP_STD = Transform.ExternalOp.V(5)
    NP_ADD = Transform.ExternalOp.V(6)
    NP_SUB = Transform.ExternalOp.V(7)
    TF_BATCH = Transform.ExternalOp.V(8)
    TF_MAP = Transform.ExternalOp.V(9)
    SK_FIT = Transform.ExternalOp.V(10)
    SK_DP_FIT = Transform.ExternalOp.V(11)
    ADD = Transform.ExternalOp.V(12)
    SUB = Transform.ExternalOp.V(13)
    MUL = Transform.ExternalOp.V(14)
    DIV = Transform.ExternalOp.V(15)
    PD_ANY = Transform.ExternalOp.V(16)
    INVERT = Transform.ExternalOp.V(17)
    GETITEM = Transform.ExternalOp.V(18)
    SK_SCALE = Transform.ExternalOp.V(19)
    LEN = Transform.ExternalOp.V(20)
    PD_PROFILE_REPORT = Transform.ExternalOp.V(21)
    PD_DESCRIBE = Transform.ExternalOp.V(22)
    PD_SELECT_DTYPES = Transform.ExternalOp.V(23)
    PD_QUANTILE = Transform.ExternalOp.V(24)
    PD_SUM = Transform.ExternalOp.V(25)
    PD_FILLNA = Transform.ExternalOp.V(26)
    PD_ROUND = Transform.ExternalOp.V(27)
    PD_RENAME = Transform.ExternalOp.V(28)
    PD_COUNT = Transform.ExternalOp.V(29)
    PD_TRANSPOSE = Transform.ExternalOp.V(30)
    PD_UNIQUE = Transform.ExternalOp.V(31)
    """class method"""

    PD_VALUE_COUNTS = Transform.ExternalOp.V(32)
    PD_TO_DICT = Transform.ExternalOp.V(33)
    PD_APPLY = Transform.ExternalOp.V(34)
    PD_SUB = Transform.ExternalOp.V(35)
    PD_MEDIAN = Transform.ExternalOp.V(36)
    PD_ABS = Transform.ExternalOp.V(37)
    PD_MAD = Transform.ExternalOp.V(38)
    PD_SKEW = Transform.ExternalOp.V(39)
    PD_KURTOSIS = Transform.ExternalOp.V(40)
    PD_AGG = Transform.ExternalOp.V(41)
    PD_DROPLEVEL = Transform.ExternalOp.V(42)
    PD_SORT_VALUES = Transform.ExternalOp.V(43)
    PD_DROP = Transform.ExternalOp.V(44)
    PD_CORR = Transform.ExternalOp.V(45)
    GT = Transform.ExternalOp.V(46)
    GE = Transform.ExternalOp.V(47)
    LT = Transform.ExternalOp.V(48)
    LE = Transform.ExternalOp.V(49)
    NE = Transform.ExternalOp.V(50)
    POS = Transform.ExternalOp.V(51)
    NEG = Transform.ExternalOp.V(52)
    ABS = Transform.ExternalOp.V(53)
    ROUND = Transform.ExternalOp.V(54)
    MOD = Transform.ExternalOp.V(55)
    OR = Transform.ExternalOp.V(56)
    AND = Transform.ExternalOp.V(57)
    SK_ONEHOT = Transform.ExternalOp.V(99)
    SK_PCA = Transform.ExternalOp.V(100)
    SK_AFFINITY_PROPAGATION = Transform.ExternalOp.V(101)
    """Cluster"""

    SK_AGGLOMERATIVE_CLUSTERING = Transform.ExternalOp.V(102)
    SK_BIRCH = Transform.ExternalOp.V(103)
    SK_DBSCAN = Transform.ExternalOp.V(104)
    SK_FEATURE_AGGLOMERATION = Transform.ExternalOp.V(105)
    SK_KMEANS = Transform.ExternalOp.V(106)
    SK_MINIBATCH_KMEANS = Transform.ExternalOp.V(107)
    SK_MEAN_SHIFT = Transform.ExternalOp.V(108)
    SK_OPTICS = Transform.ExternalOp.V(109)
    SK_SPECTRAL_CLUSTERING = Transform.ExternalOp.V(110)
    SK_SPECTRAL_BICLUSTERING = Transform.ExternalOp.V(58)
    SK_SPECTRAL_COCLUSTERING = Transform.ExternalOp.V(59)
    SK_ADABOOST_CLASSIFIER = Transform.ExternalOp.V(60)
    """ensemble"""

    SK_ADABOOST_REGRESSOR = Transform.ExternalOp.V(61)
    SK_BAGGING_CLASSIFIER = Transform.ExternalOp.V(62)
    SK_BAGGING_REGRESSOR = Transform.ExternalOp.V(63)
    SK_EXTRA_TREES_REGRESSOR = Transform.ExternalOp.V(64)
    SK_EXTRA_TREES_CLASSIFIER = Transform.ExternalOp.V(65)
    SK_GRADIENT_BOOSTING_CLASSIFIER = Transform.ExternalOp.V(66)
    SK_GRADIENT_BOOSTING_REGRESSOR = Transform.ExternalOp.V(67)
    SK_ISOLATION_FOREST = Transform.ExternalOp.V(68)
    SK_RANDOM_FOREST_CLASSIFIER = Transform.ExternalOp.V(69)
    SK_RANDOM_FOREST_REGRESSOR = Transform.ExternalOp.V(70)
    SK_RANDOM_TREES_EMBEDDING = Transform.ExternalOp.V(71)
    SK_STACKING_CLASSIFIER = Transform.ExternalOp.V(72)
    SK_STACKING_REGRESSOR = Transform.ExternalOp.V(73)
    SK_VOTING_CLASSIFIER = Transform.ExternalOp.V(74)
    SK_VOTING_REGRESSOR = Transform.ExternalOp.V(75)
    SK_HIST_GRADIENT_BOOSTING_REGRESSOR = Transform.ExternalOp.V(76)
    SK_HIST_GRADIENT_BOOSTING_CLASSIFIER = Transform.ExternalOp.V(77)
    SK_CROSS_VAL_SCORE = Transform.ExternalOp.V(78)
    """Model selection"""

    SK_TRAIN_TEST_SPLIT = Transform.ExternalOp.V(79)
    SK_REPEATED_STRATIFIED_KFOLD = Transform.ExternalOp.V(80)
    SK_ACCURACY_SCORE = Transform.ExternalOp.V(81)
    """Metrics"""

    SK_AVERAGE_PRECISION_SCORE = Transform.ExternalOp.V(82)
    SK_CLASSIFICATION_REPORT = Transform.ExternalOp.V(83)
    SK_CONFUSION_MATRIX = Transform.ExternalOp.V(84)
    SK_F1_SCORE = Transform.ExternalOp.V(85)
    SK_PLOT_CONFUSION_MATRIX = Transform.ExternalOp.V(86)
    SK_PRECISION_RECALL_CURVE = Transform.ExternalOp.V(87)
    SK_PRECISION_SCORE = Transform.ExternalOp.V(88)
    SK_RECALL_SCORE = Transform.ExternalOp.V(89)
    SK_ROC_AUC_SCORE = Transform.ExternalOp.V(90)
    SK_ROC_CURVE = Transform.ExternalOp.V(91)
    XGB_CLASSIFIER = Transform.ExternalOp.V(92)
    """xgb"""

    XGB_CLASSIFIER_PREDICT = Transform.ExternalOp.V(93)
    PD_GET_DUMMIES = Transform.ExternalOp.V(94)
    """Pandas"""

    PD_JOIN = Transform.ExternalOp.V(95)
    PD_CONCAT = Transform.ExternalOp.V(96)
    SK_RANDOM_FOREST_CLASSIFIER_PREDICT = Transform.ExternalOp.V(97)
    """ensemble predict"""

    SK_LABEL_ENCODER_FIT_TRANSFORM = Transform.ExternalOp.V(98)

    class PropertiesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text = ...
        value: typing.Text = ...
        def __init__(self,
            *,
            key : typing.Text = ...,
            value : typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    class Spec(google.protobuf.message.Message):
        """Definitions"""
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        IDENTITY_FIELD_NUMBER: builtins.int
        VARIABLE_FIELD_NUMBER: builtins.int
        COMPOSED_FIELD_NUMBER: builtins.int
        PROJECT_FIELD_NUMBER: builtins.int
        FILTER_FIELD_NUMBER: builtins.int
        SHUFFLE_FIELD_NUMBER: builtins.int
        JOIN_FIELD_NUMBER: builtins.int
        CAST_FIELD_NUMBER: builtins.int
        SAMPLE_FIELD_NUMBER: builtins.int
        USER_SETTINGS_FIELD_NUMBER: builtins.int
        PROTECT_DATASET_FIELD_NUMBER: builtins.int
        EXTERNAL_FIELD_NUMBER: builtins.int
        SYNTHETIC_FIELD_NUMBER: builtins.int
        TRANSCODE_FIELD_NUMBER: builtins.int
        INVERSE_TRANSCODE_FIELD_NUMBER: builtins.int
        @property
        def identity(self) -> global___Transform.Identity: ...
        @property
        def variable(self) -> global___Transform.Variable: ...
        @property
        def composed(self) -> global___Transform.Composed: ...
        @property
        def project(self) -> global___Transform.Project: ...
        @property
        def filter(self) -> global___Transform.Filter: ...
        @property
        def shuffle(self) -> global___Transform.Shuffle: ...
        @property
        def join(self) -> global___Transform.Join: ...
        @property
        def cast(self) -> global___Transform.Cast: ...
        @property
        def sample(self) -> global___Transform.Sample: ...
        @property
        def user_settings(self) -> global___Transform.UserSettings: ...
        @property
        def protect_dataset(self) -> global___Transform.Protect: ...
        @property
        def external(self) -> global___Transform.External:
            """np transforms, pd transforms,..."""
            pass
        @property
        def synthetic(self) -> global___Transform.Synthetic: ...
        @property
        def transcode(self) -> global___Transform.Transcode: ...
        @property
        def inverse_transcode(self) -> global___Transform.InverseTranscode: ...
        def __init__(self,
            *,
            identity : typing.Optional[global___Transform.Identity] = ...,
            variable : typing.Optional[global___Transform.Variable] = ...,
            composed : typing.Optional[global___Transform.Composed] = ...,
            project : typing.Optional[global___Transform.Project] = ...,
            filter : typing.Optional[global___Transform.Filter] = ...,
            shuffle : typing.Optional[global___Transform.Shuffle] = ...,
            join : typing.Optional[global___Transform.Join] = ...,
            cast : typing.Optional[global___Transform.Cast] = ...,
            sample : typing.Optional[global___Transform.Sample] = ...,
            user_settings : typing.Optional[global___Transform.UserSettings] = ...,
            protect_dataset : typing.Optional[global___Transform.Protect] = ...,
            external : typing.Optional[global___Transform.External] = ...,
            synthetic : typing.Optional[global___Transform.Synthetic] = ...,
            transcode : typing.Optional[global___Transform.Transcode] = ...,
            inverse_transcode : typing.Optional[global___Transform.InverseTranscode] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"cast",b"cast",u"composed",b"composed",u"external",b"external",u"filter",b"filter",u"identity",b"identity",u"inverse_transcode",b"inverse_transcode",u"join",b"join",u"project",b"project",u"protect_dataset",b"protect_dataset",u"sample",b"sample",u"shuffle",b"shuffle",u"spec",b"spec",u"synthetic",b"synthetic",u"transcode",b"transcode",u"user_settings",b"user_settings",u"variable",b"variable"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"cast",b"cast",u"composed",b"composed",u"external",b"external",u"filter",b"filter",u"identity",b"identity",u"inverse_transcode",b"inverse_transcode",u"join",b"join",u"project",b"project",u"protect_dataset",b"protect_dataset",u"sample",b"sample",u"shuffle",b"shuffle",u"spec",b"spec",u"synthetic",b"synthetic",u"transcode",b"transcode",u"user_settings",b"user_settings",u"variable",b"variable"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal[u"spec",b"spec"]) -> typing.Optional[typing_extensions.Literal["identity","variable","composed","project","filter","shuffle","join","cast","sample","user_settings","protect_dataset","external","synthetic","transcode","inverse_transcode"]]: ...

    class External(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        ARGUMENTS_FIELD_NUMBER: builtins.int
        NAMED_ARGUMENTS_FIELD_NUMBER: builtins.int
        OP_FIELD_NUMBER: builtins.int
        arguments: builtins.bytes = ...
        named_arguments: builtins.bytes = ...
        op: global___Transform.ExternalOp.V = ...
        def __init__(self,
            *,
            arguments : builtins.bytes = ...,
            named_arguments : builtins.bytes = ...,
            op : global___Transform.ExternalOp.V = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"arguments",b"arguments",u"named_arguments",b"named_arguments",u"op",b"op"]) -> None: ...

    class Identity(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        def __init__(self,
            ) -> None: ...

    class Variable(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        POSITION_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        position: builtins.int = ...
        name: typing.Text = ...
        def __init__(self,
            *,
            position : builtins.int = ...,
            name : typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"name",b"name",u"position",b"position"]) -> None: ...

    class Composed(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class NamedArgumentsEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: typing.Text = ...
            value: typing.Text = ...
            def __init__(self,
                *,
                key : typing.Text = ...,
                value : typing.Text = ...,
                ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key",u"value",b"value"]) -> None: ...

        TRANSFORM_FIELD_NUMBER: builtins.int
        ARGUMENTS_FIELD_NUMBER: builtins.int
        NAMED_ARGUMENTS_FIELD_NUMBER: builtins.int
        transform: typing.Text = ...
        """Transform"""

        @property
        def arguments(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
            """Arguments of the current transform are transforms"""
            pass
        @property
        def named_arguments(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]: ...
        def __init__(self,
            *,
            transform : typing.Text = ...,
            arguments : typing.Optional[typing.Iterable[typing.Text]] = ...,
            named_arguments : typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"arguments",b"arguments",u"named_arguments",b"named_arguments",u"transform",b"transform"]) -> None: ...

    class Project(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        PROJECTION_FIELD_NUMBER: builtins.int
        @property
        def projection(self) -> sarus_data_spec.protobuf.type_pb2.Type:
            """This should be a 'supertype' the type the data can project into."""
            pass
        def __init__(self,
            *,
            projection : typing.Optional[sarus_data_spec.protobuf.type_pb2.Type] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"projection",b"projection"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"projection",b"projection"]) -> None: ...

    class Filter(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        FILTER_FIELD_NUMBER: builtins.int
        @property
        def filter(self) -> sarus_data_spec.protobuf.type_pb2.Type:
            """This should be a 'subtype' the type the data can be retricted to."""
            pass
        def __init__(self,
            *,
            filter : typing.Optional[sarus_data_spec.protobuf.type_pb2.Type] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"filter",b"filter"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"filter",b"filter"]) -> None: ...

    class Shuffle(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        def __init__(self,
            ) -> None: ...

    class Join(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        ON_FIELD_NUMBER: builtins.int
        @property
        def on(self) -> sarus_data_spec.protobuf.type_pb2.Type:
            """This should be a common 'supertype' between tables."""
            pass
        def __init__(self,
            *,
            on : typing.Optional[sarus_data_spec.protobuf.type_pb2.Type] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"on",b"on"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"on",b"on"]) -> None: ...

    class Cast(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TYPE_FIELD_NUMBER: builtins.int
        @property
        def type(self) -> sarus_data_spec.protobuf.type_pb2.Type:
            """Type to cast into."""
            pass
        def __init__(self,
            *,
            type : typing.Optional[sarus_data_spec.protobuf.type_pb2.Type] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> None: ...

    class Sample(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        FRACTION_FIELD_NUMBER: builtins.int
        SIZE_FIELD_NUMBER: builtins.int
        fraction: builtins.float = ...
        size: builtins.int = ...
        def __init__(self,
            *,
            fraction : builtins.float = ...,
            size : builtins.int = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"fraction",b"fraction",u"proportion",b"proportion",u"size",b"size"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"fraction",b"fraction",u"proportion",b"proportion",u"size",b"size"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal[u"proportion",b"proportion"]) -> typing.Optional[typing_extensions.Literal["fraction","size"]]: ...

    class SchemaInference(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class CastPolicy(_CastPolicy, metaclass=_CastPolicyEnumTypeWrapper):
            pass
        class _CastPolicy:
            V = typing.NewType('V', builtins.int)
        class _CastPolicyEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CastPolicy.V], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
            NONE = Transform.SchemaInference.CastPolicy.V(0)
            MOST_LIKELY = Transform.SchemaInference.CastPolicy.V(1)

        NONE = Transform.SchemaInference.CastPolicy.V(0)
        MOST_LIKELY = Transform.SchemaInference.CastPolicy.V(1)

        CAST_POLICY_FIELD_NUMBER: builtins.int
        cast_policy: global___Transform.SchemaInference.CastPolicy.V = ...
        def __init__(self,
            *,
            cast_policy : global___Transform.SchemaInference.CastPolicy.V = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"cast_policy",b"cast_policy"]) -> None: ...

    class GroupBy(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        KEY_FIELD_NUMBER: builtins.int
        key: typing.Text = ...
        def __init__(self,
            *,
            key : typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key"]) -> None: ...

    class Synthetic(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        def __init__(self,
            ) -> None: ...

    class UserSettings(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TYPE_FIELD_NUMBER: builtins.int
        @property
        def type(self) -> sarus_data_spec.protobuf.type_pb2.Type: ...
        def __init__(self,
            *,
            type : typing.Optional[sarus_data_spec.protobuf.type_pb2.Type] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> None: ...

    class Protect(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        PROTECTED_ENTITIES_FIELD_NUMBER: builtins.int
        PUBLIC_ENTITIES_FIELD_NUMBER: builtins.int
        @property
        def protected_entities(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[sarus_data_spec.protobuf.path_pb2.Path]: ...
        @property
        def public_entities(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[sarus_data_spec.protobuf.path_pb2.Path]: ...
        def __init__(self,
            *,
            protected_entities : typing.Optional[typing.Iterable[sarus_data_spec.protobuf.path_pb2.Path]] = ...,
            public_entities : typing.Optional[typing.Iterable[sarus_data_spec.protobuf.path_pb2.Path]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"protected_entities",b"protected_entities",u"public_entities",b"public_entities"]) -> None: ...

    class Transcode(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        STATISTICS_FIELD_NUMBER: builtins.int
        DATA_TYPE_FIELD_NUMBER: builtins.int
        PROTECTED_PATH_FIELD_NUMBER: builtins.int
        @property
        def statistics(self) -> sarus_data_spec.protobuf.statistics_pb2.Statistics: ...
        @property
        def data_type(self) -> sarus_data_spec.protobuf.type_pb2.Type: ...
        @property
        def protected_path(self) -> sarus_data_spec.protobuf.path_pb2.Path: ...
        def __init__(self,
            *,
            statistics : typing.Optional[sarus_data_spec.protobuf.statistics_pb2.Statistics] = ...,
            data_type : typing.Optional[sarus_data_spec.protobuf.type_pb2.Type] = ...,
            protected_path : typing.Optional[sarus_data_spec.protobuf.path_pb2.Path] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"data_type",b"data_type",u"protected_path",b"protected_path",u"statistics",b"statistics"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"data_type",b"data_type",u"protected_path",b"protected_path",u"statistics",b"statistics"]) -> None: ...

    class InverseTranscode(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TRANSCODE_FIELD_NUMBER: builtins.int
        @property
        def transcode(self) -> global___Transform.Transcode: ...
        def __init__(self,
            *,
            transcode : typing.Optional[global___Transform.Transcode] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"transcode",b"transcode"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"transcode",b"transcode"]) -> None: ...

    UUID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DOC_FIELD_NUMBER: builtins.int
    SPEC_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    INVERSIBLE_FIELD_NUMBER: builtins.int
    SCHEMA_PRESERVING_FIELD_NUMBER: builtins.int
    uuid: typing.Text = ...
    """A dataset transform
    e.g. RFC 4122 id used to refer to the transform
    """

    name: typing.Text = ...
    doc: typing.Text = ...
    @property
    def spec(self) -> global___Transform.Spec: ...
    @property
    def properties(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]:
        """Other properties"""
        pass
    inversible: builtins.bool = ...
    schema_preserving: builtins.bool = ...
    def __init__(self,
        *,
        uuid : typing.Text = ...,
        name : typing.Text = ...,
        doc : typing.Text = ...,
        spec : typing.Optional[global___Transform.Spec] = ...,
        properties : typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
        inversible : builtins.bool = ...,
        schema_preserving : builtins.bool = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"spec",b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"doc",b"doc",u"inversible",b"inversible",u"name",b"name",u"properties",b"properties",u"schema_preserving",b"schema_preserving",u"spec",b"spec",u"uuid",b"uuid"]) -> None: ...
global___Transform = Transform



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class Attribute(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    class PropertiesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text = ...
        value: typing.Text = ...
        def __init__(self,
            *,
            key : typing.Text = ...,
            value : typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    UUID_FIELD_NUMBER: builtins.int
    OBJECT_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    uuid: typing.Text = ...
    """General attribute about an object"""

    object: typing.Text = ...
    """uuid of an object"""

    @property
    def properties(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]:
        """Other properties"""
        pass
    def __init__(self,
        *,
        uuid : typing.Text = ...,
        object : typing.Text = ...,
        properties : typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"object",b"object",u"properties",b"properties",u"uuid",b"uuid"]) -> None: ...
global___Attribute = Attribute



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/schema.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from sarus_data_spec.protobuf import type_pb2 as sarus__data__spec_dot_protobuf_dot_type__pb2
from sarus_data_spec.protobuf import path_pb2 as sarus__data__spec_dot_protobuf_dot_path__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/schema.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n%sarus_data_spec/protobuf/schema.proto\x12\x0fsarus_data_spec\x1a#sarus_data_spec/protobuf/type.proto\x1a#sarus_data_spec/protobuf/path.proto\"\xd0\x03\n\x06Schema\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x0f\n\x07\x64\x61taset\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12#\n\x04type\x18\x04 \x01(\x0b\x32\x15.sarus_data_spec.Type\x12(\n\tprotected\x18\x05 \x01(\x0b\x32\x15.sarus_data_spec.Path\x12;\n\nproperties\x18\x06 \x03(\x0b\x32\'.sarus_data_spec.Schema.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\xd9\x01\n\nHypothesis\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x0f\n\x07\x64\x61taset\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12#\n\x04type\x18\x04 \x01(\x0b\x32\x15.sarus_data_spec.Type\x12\x46\n\nproperties\x18\x05 \x03(\x0b\x32\x32.sarus_data_spec.Schema.Hypothesis.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x62\x06proto3'
  ,
  dependencies=[sarus__data__spec_dot_protobuf_dot_type__pb2.DESCRIPTOR,sarus__data__spec_dot_protobuf_dot_path__pb2.DESCRIPTOR,])




_SCHEMA_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Schema.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Schema.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Schema.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=328,
  serialized_end=377,
)

_SCHEMA_HYPOTHESIS_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Schema.Hypothesis.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Schema.Hypothesis.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Schema.Hypothesis.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=328,
  serialized_end=377,
)

_SCHEMA_HYPOTHESIS = _descriptor.Descriptor(
  name='Hypothesis',
  full_name='sarus_data_spec.Schema.Hypothesis',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='sarus_data_spec.Schema.Hypothesis.uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dataset', full_name='sarus_data_spec.Schema.Hypothesis.dataset', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Schema.Hypothesis.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='sarus_data_spec.Schema.Hypothesis.type', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Schema.Hypothesis.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SCHEMA_HYPOTHESIS_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=380,
  serialized_end=597,
)

_SCHEMA = _descriptor.Descriptor(
  name='Schema',
  full_name='sarus_data_spec.Schema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='sarus_data_spec.Schema.uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dataset', full_name='sarus_data_spec.Schema.dataset', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Schema.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='sarus_data_spec.Schema.type', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='protected', full_name='sarus_data_spec.Schema.protected', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Schema.properties', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SCHEMA_PROPERTIESENTRY, _SCHEMA_HYPOTHESIS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=133,
  serialized_end=597,
)

_SCHEMA_PROPERTIESENTRY.containing_type = _SCHEMA
_SCHEMA_HYPOTHESIS_PROPERTIESENTRY.containing_type = _SCHEMA_HYPOTHESIS
_SCHEMA_HYPOTHESIS.fields_by_name['type'].message_type = sarus__data__spec_dot_protobuf_dot_type__pb2._TYPE
_SCHEMA_HYPOTHESIS.fields_by_name['properties'].message_type = _SCHEMA_HYPOTHESIS_PROPERTIESENTRY
_SCHEMA_HYPOTHESIS.containing_type = _SCHEMA
_SCHEMA.fields_by_name['type'].message_type = sarus__data__spec_dot_protobuf_dot_type__pb2._TYPE
_SCHEMA.fields_by_name['protected'].message_type = sarus__data__spec_dot_protobuf_dot_path__pb2._PATH
_SCHEMA.fields_by_name['properties'].message_type = _SCHEMA_PROPERTIESENTRY
DESCRIPTOR.message_types_by_name['Schema'] = _SCHEMA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Schema = _reflection.GeneratedProtocolMessageType('Schema', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _SCHEMA_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.schema_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Schema.PropertiesEntry)
    })
  ,

  'Hypothesis' : _reflection.GeneratedProtocolMessageType('Hypothesis', (_message.Message,), {

    'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
      'DESCRIPTOR' : _SCHEMA_HYPOTHESIS_PROPERTIESENTRY,
      '__module__' : 'sarus_data_spec.protobuf.schema_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Schema.Hypothesis.PropertiesEntry)
      })
    ,
    'DESCRIPTOR' : _SCHEMA_HYPOTHESIS,
    '__module__' : 'sarus_data_spec.protobuf.schema_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Schema.Hypothesis)
    })
  ,
  'DESCRIPTOR' : _SCHEMA,
  '__module__' : 'sarus_data_spec.protobuf.schema_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.Schema)
  })
_sym_db.RegisterMessage(Schema)
_sym_db.RegisterMessage(Schema.PropertiesEntry)
_sym_db.RegisterMessage(Schema.Hypothesis)
_sym_db.RegisterMessage(Schema.Hypothesis.PropertiesEntry)


_SCHEMA_PROPERTIESENTRY._options = None
_SCHEMA_HYPOTHESIS_PROPERTIESENTRY._options = None
# @@protoc_insertion_point(module_scope)



from typing import Protocol, runtime_checkable

from sarus_data_spec.manager.typing import Manager
from sarus_data_spec.storage.typing import Storage
from sarus_data_spec.typing import Factory


@runtime_checkable
class Context(Protocol):
    """Provide the shared services."""

    def factory(self) -> Factory:
        """Provide a factory to create python objects from their protobuf
        counterparts."""
        ...

    def storage(self) -> Storage:
        """Provide the storage device for object persistence."""
        ...

    def manager(self) -> Manager:
        """Provide the manager to implement object behaviors."""
        ...


@runtime_checkable
class HasContext(Protocol):
    """Has a context."""

    def context(self) -> Context:
        """Return a context (usually a singleton)."""
        ...



from functools import wraps
from typing import Any, Callable, cast

import sarus_data_spec.protobuf as sp
import sarus_data_spec.typing as st


async def pandas_or_value(dataspec: st.DataSpec) -> Any:
    if dataspec.prototype() == sp.Dataset:
        dataset = cast(st.Dataset, dataspec)
        return await dataset.async_to_pandas()
    else:
        scalar = cast(st.Scalar, dataspec)
        return await scalar.async_value()


def two_arguments_ops(ops_fn: Callable) -> Callable:
    """Unwrap parent DataSpecs values before applying the ops.

    The wrapped ops should have exactly two arguments, either dataspecs parents
    or in the Python objects arguments.
    """

    @wraps(ops_fn)
    async def wrapped_ops_fn(
        dataspec: st.DataSpec, *args: Any, invert: bool = False, **kwargs: Any
    ) -> Any:
        flat_args = list(args) + list(kwargs.values())
        arg_parents, kwarg_parents = dataspec.parents()
        flat_parents = list(arg_parents) + list(kwarg_parents.values())

        # We should have two items to add with at least a DataSpec
        assert (
            len(flat_parents) + len(flat_args) == 2
        ), 'Number of arguments differs from 2'
        assert len(flat_parents) >= 1, 'No flat parent'

        if len(flat_args) == 1:
            (parent,) = flat_parents
            val_1 = await pandas_or_value(parent)
            (val_2,) = flat_args

        else:
            (parent_1, parent_2) = flat_parents
            val_1 = await pandas_or_value(parent_1)
            val_2 = await pandas_or_value(parent_2)

        if invert:
            val_1, val_2 = val_2, val_1

        return await ops_fn(val_1, val_2)

    return wrapped_ops_fn


def one_parent_ops(ops_fn: Callable) -> Callable:
    """Check and unwrap an ops arguments.

    Wrapped ops have exactly one parent DataSpec.
    """

    @wraps(ops_fn)
    async def wrapped_ops_fn(
        dataspec: st.DataSpec, *args: Any, **kwargs: Any
    ) -> Any:
        arg_parents, kwarg_parents = dataspec.parents()
        flat_parents = list(arg_parents) + list(kwarg_parents.values())
        assert len(flat_parents) == 1
        (parent,) = flat_parents
        parent_val = await pandas_or_value(parent)
        return await ops_fn(parent_val, *args, **kwargs)

    return wrapped_ops_fn



from __future__ import annotations

from typing import Any, cast

try:
    from sklearn import (
        cluster,
        decomposition,
        ensemble,
        metrics,
        model_selection,
        preprocessing,
        svm,
    )
except ModuleNotFoundError:
    pass  # error message in typing.py

import sarus_data_spec.typing as st

from .utils import one_parent_ops, pandas_or_value


async def sk_fit(  # to be renamed ?
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> svm.SVC:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, svm.SVC)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


@one_parent_ops
async def sk_scale(val: Any, *args: Any, **kwargs: Any) -> Any:
    return preprocessing.scale(val, *args, **kwargs)


async def sk_onehot(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> preprocessing.OneHotEncoder:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, preprocessing.OneHotEncoder)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_pca(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> decomposition.PCA:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, decomposition.PCA)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


# cluster
async def sk_affinity_propagation(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.AffinityPropagation:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.AffinityPropagation)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_agglomerative_clustering(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.AgglomerativeClustering:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.AgglomerativeClustering)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_birch(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.Birch:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.Birch)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_dbscan(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.DBSCAN:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.DBSCAN)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_feature_agglomeration(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.FeatureAgglomeration:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.FeatureAgglomeration)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_kmeans(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.KMeans:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.KMeans)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_minibatch_kmeans(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.MiniBatchKMeans:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.MiniBatchKMeans)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_mean_shift(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.MeanShift:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.MeanShift)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_optics(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.OPTICS:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.OPTICS)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_spectral_clustering(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.SpectralClustering:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.SpectralClustering)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_spectral_biclustering(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.SpectralBiclustering:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.SpectralBiclustering)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


async def sk_spectral_coclustering(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> cluster.SpectralCoclustering:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, cluster.SpectralCoclustering)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


# ensemble
async def sk_adaboost_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.AdaBoostClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.AdaBoostClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_adaboost_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.AdaBoostRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.AdaBoostRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_bagging_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.BaggingClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.BaggingClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_bagging_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.BaggingRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.BaggingRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_extra_trees_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.ExtraTreesClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.ExtraTreesClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_extra_trees_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.ExtraTreesRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.ExtraTreesRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_gradient_boosting_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.GradientBoostingClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.GradientBoostingClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_gradient_boosting_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.GradientBoostingRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.GradientBoostingRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_isolation_forest(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.IsolationForest:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.IsolationForest)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_random_forest_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.RandomForestClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.RandomForestClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_random_forest_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.RandomForestRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.RandomForestRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_random_trees_embedding(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.RandomTreesEmbedding:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.RandomTreesEmbedding)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_stacking_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.StackingClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.StackingClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_stacking_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.StackingRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.StackingRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_voting_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.VotingClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.VotingClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_voting_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.VotingRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.VotingRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_hist_gradient_boosting_classifier(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.HistGradientBoostingClassifier:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.HistGradientBoostingClassifier)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_hist_gradient_boosting_regressor(
    scalar: st.Scalar, *args: Any, **kwargs: Any
) -> ensemble.HistGradientBoostingRegressor:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.HistGradientBoostingRegressor)
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    fitted_model = model.fit(X, y, *args, **kwargs)
    return fitted_model


async def sk_cross_val_score(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (ds_model, X, y), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    return model_selection.cross_val_score(model, X, y, *args, **kwargs)


async def sk_train_test_split(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (X, y), _ = scalar.parents()
    X = await pandas_or_value(X)
    y = await pandas_or_value(y)
    return model_selection.train_test_split(X, y, *args, **kwargs)


async def sk_repeated_stratified_kfold(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, model_selection.RepeatedStratifiedKFold)
    X = await pandas_or_value(X)
    fitted_model = model.fit(X, *args, **kwargs)
    return fitted_model


# metrics
async def sk_accuracy_score(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.accuracy_score(y_true, y_pred, *args, **kwargs)


async def sk_average_precision_score(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.average_precision_score(y_true, y_pred, *args, **kwargs)


async def sk_classification_report(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.classification_report(y_true, y_pred, *args, **kwargs)


async def sk_confusion_matrix(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.confusion_matrix(y_true, y_pred, *args, **kwargs)


async def sk_f1_score(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.f1_score(y_true, y_pred, *args, **kwargs)


async def sk_plot_confusion_matrix(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (ds_model, X, y_true), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    X = await pandas_or_value(X)
    y_true = await pandas_or_value(y_true)
    return metrics.plot_confusion_matrix(model, X, y_true, *args, **kwargs)


async def sk_precision_recall_curve(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_scores), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_scores = await pandas_or_value(y_scores)
    return metrics.precision_recall_curve(y_true, y_scores, *args, **kwargs)


async def sk_precision_score(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.precision_score(y_true, y_pred, *args, **kwargs)


async def sk_recall_score(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.recall_score(y_true, y_pred, *args, **kwargs)


async def sk_roc_auc_score(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_pred), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_pred = await pandas_or_value(y_pred)
    return metrics.roc_auc_score(y_true, y_pred, *args, **kwargs)


async def sk_roc_curve(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (y_true, y_scores), _ = scalar.parents()
    y_true = await pandas_or_value(y_true)
    y_scores = await pandas_or_value(y_scores)
    return metrics.roc_curve(y_true, y_scores, *args, **kwargs)


async def sk_random_forest_classifier_predict(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, ensemble.RandomForestClassifier)
    X = await pandas_or_value(X)
    y_pred = model.predict(X, *args, **kwargs)
    return y_pred


async def sk_label_encoder_fit_transform(
    scalar: st.Scalar,
    *args: Any,
    **kwargs: Any,
) -> Any:
    (ds_model, X), _ = scalar.parents()
    model = await cast(st.Scalar, ds_model).async_value()
    assert isinstance(model, preprocessing.LabelEncoder)
    X = await pandas_or_value(X)
    X_transformed = model.fit_transform(X, *args, **kwargs)
    return X_transformed



from __future__ import annotations

from typing import Any, Dict
import pickle as pkl

try:
    from sklearn import (
        cluster,
        decomposition,
        ensemble,
        model_selection,
        preprocessing,
        svm,
    )
    import xgboost  # type: ignore[import]
except ModuleNotFoundError:
    pass  # error message in typing.py

import sarus_data_spec.protobuf as sp
import sarus_data_spec.typing as st


async def create_model(scalar: st.Scalar) -> svm.SVC:
    model_spec = scalar.protobuf().spec.model

    args = pkl.loads(model_spec.arguments)
    kwargs = pkl.loads(model_spec.named_arguments)

    model_mapping: Dict[sp.Scalar.Model.ModelClass.V, Any] = {
        sp.Scalar.Model.ModelClass.SK_SVC: svm.SVC,
        sp.Scalar.Model.ModelClass.SK_ONEHOT: preprocessing.OneHotEncoder,
        sp.Scalar.Model.ModelClass.SK_LABEL_ENCODER: preprocessing.LabelEncoder,
        sp.Scalar.Model.ModelClass.SK_PCA: decomposition.PCA,
        # cluster
        sp.Scalar.Model.ModelClass.SK_AFFINITY_PROPAGATION: cluster.AffinityPropagation,
        sp.Scalar.Model.ModelClass.SK_AGGLOMERATIVE_CLUSTERING: cluster.AgglomerativeClustering,
        sp.Scalar.Model.ModelClass.SK_BIRCH: cluster.Birch,
        sp.Scalar.Model.ModelClass.SK_DBSCAN: cluster.DBSCAN,
        sp.Scalar.Model.ModelClass.SK_FEATURE_AGGLOMERATION: cluster.FeatureAgglomeration,
        sp.Scalar.Model.ModelClass.SK_KMEANS: cluster.KMeans,
        sp.Scalar.Model.ModelClass.SK_MINIBATCH_KMEANS: cluster.MiniBatchKMeans,
        sp.Scalar.Model.ModelClass.SK_MEAN_SHIFT: cluster.MeanShift,
        sp.Scalar.Model.ModelClass.SK_OPTICS: cluster.OPTICS,
        sp.Scalar.Model.ModelClass.SK_SPECTRAL_CLUSTERING: cluster.SpectralClustering,
        sp.Scalar.Model.ModelClass.SK_SPECTRAL_BICLUSTERING: cluster.SpectralBiclustering,
        sp.Scalar.Model.ModelClass.SK_SPECTRAL_COCLUSTERING: cluster.SpectralCoclustering,
        # ensemble
        sp.Scalar.Model.ModelClass.SK_ADABOOST_CLASSIFIER: ensemble.AdaBoostClassifier,
        sp.Scalar.Model.ModelClass.SK_ADABOOST_REGRESSOR: ensemble.AdaBoostRegressor,
        sp.Scalar.Model.ModelClass.SK_BAGGING_CLASSIFIER: ensemble.BaggingClassifier,
        sp.Scalar.Model.ModelClass.SK_BAGGING_REGRESSOR: ensemble.BaggingRegressor,
        sp.Scalar.Model.ModelClass.SK_EXTRA_TREES_CLASSIFIER: ensemble.ExtraTreesClassifier,
        sp.Scalar.Model.ModelClass.SK_EXTRA_TREES_REGRESSOR: ensemble.ExtraTreesRegressor,
        sp.Scalar.Model.ModelClass.SK_GRADIENT_BOOSTING_CLASSIFIER: ensemble.GradientBoostingClassifier,
        sp.Scalar.Model.ModelClass.SK_GRADIENT_BOOSTING_REGRESSOR: ensemble.GradientBoostingRegressor,
        sp.Scalar.Model.ModelClass.SK_ISOLATION_FOREST: ensemble.IsolationForest,
        sp.Scalar.Model.ModelClass.SK_RANDOM_FOREST_CLASSIFIER: ensemble.RandomForestClassifier,
        sp.Scalar.Model.ModelClass.SK_RANDOM_FOREST_REGRESSOR: ensemble.RandomForestRegressor,
        sp.Scalar.Model.ModelClass.SK_RANDOM_TREES_EMBEDDING: ensemble.RandomTreesEmbedding,
        sp.Scalar.Model.ModelClass.SK_STACKING_CLASSIFIER: ensemble.StackingClassifier,
        sp.Scalar.Model.ModelClass.SK_STACKING_REGRESSOR: ensemble.StackingRegressor,
        sp.Scalar.Model.ModelClass.SK_VOTING_CLASSIFIER: ensemble.VotingClassifier,
        sp.Scalar.Model.ModelClass.SK_VOTING_REGRESSOR: ensemble.VotingRegressor,
        sp.Scalar.Model.ModelClass.SK_HIST_GRADIENT_BOOSTING_CLASSIFIER: ensemble.HistGradientBoostingClassifier,
        sp.Scalar.Model.ModelClass.SK_HIST_GRADIENT_BOOSTING_REGRESSOR: ensemble.HistGradientBoostingRegressor,
        # model_selection
        sp.Scalar.Model.ModelClass.SK_REPEATED_STRATIFIED_KFOLD: model_selection.RepeatedStratifiedKFold,
        # xgb
        sp.Scalar.Model.ModelClass.XGB_CLASSIFIER: xgboost.XGBClassifier,
    }
    ModelClass = model_mapping[model_spec.model_class]

    return ModelClass(*args, **kwargs)



from typing import Any, List, Tuple, Union
import logging

import pandas as pd

from .utils import one_parent_ops, two_arguments_ops

logger = logging.getLogger(__name__)


@one_parent_ops
async def pd_loc(
    parent_val: Any, key: Tuple[Union[str, slice, List[str]], ...]
) -> pd.DataFrame:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return pd.core.indexing._LocIndexer.__getitem__(parent_val.loc, key)


@two_arguments_ops
async def pd_eq(val_1: Any, val_2: Any) -> pd.DataFrame:
    return val_1 == val_2


@one_parent_ops
async def pd_mean(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.mean(*args, **kwargs)


@one_parent_ops
async def pd_std(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.std(*args, **kwargs)


@one_parent_ops
async def pd_any(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.any(*args, **kwargs)


@one_parent_ops
async def pd_describe(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.describe(*args, **kwargs)


@one_parent_ops
async def pd_select_dtypes(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.select_dtypes(*args, **kwargs)


@one_parent_ops
async def pd_quantile(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.quantile(*args, **kwargs)


@one_parent_ops
async def pd_sum(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.sum(*args, **kwargs)


@one_parent_ops
async def pd_fillna(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.fillna(*args, **kwargs)


@one_parent_ops
async def pd_round(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.round(*args, **kwargs)


@one_parent_ops
async def pd_rename(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.rename(*args, **kwargs)


@one_parent_ops
async def pd_count(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.count(*args, **kwargs)


@one_parent_ops
async def pd_transpose(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.transpose(*args, **kwargs)


@one_parent_ops
async def pd_unique(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.unique(*args, **kwargs)


@one_parent_ops
async def pd_value_counts(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.value_counts(*args, **kwargs)


@one_parent_ops
async def pd_to_dict(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.to_dict(*args, **kwargs)


@one_parent_ops
async def pd_apply(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.apply(*args, **kwargs)


@one_parent_ops
async def pd_median(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.median(*args, **kwargs)


@one_parent_ops
async def pd_abs(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.abs(*args, **kwargs)


@one_parent_ops
async def pd_mad(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.mad(*args, **kwargs)


@one_parent_ops
async def pd_skew(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.skew(*args, **kwargs)


@one_parent_ops
async def pd_kurtosis(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.kurtosis(*args, **kwargs)


@one_parent_ops
async def pd_agg(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.agg(*args, **kwargs)


@one_parent_ops
async def pd_droplevel(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.droplevel(*args, **kwargs)


@one_parent_ops
async def pd_sort_values(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.sort_values(*args, **kwargs)


@one_parent_ops
async def pd_drop(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.drop(*args, **kwargs)


@one_parent_ops
async def pd_corr(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return parent_val.corr(*args, **kwargs)


@one_parent_ops
async def pd_get_dummies(parent_val: Any, *args: Any, **kwargs: Any) -> Any:
    assert type(parent_val) in [pd.Series, pd.DataFrame]
    return pd.get_dummies(parent_val, *args, **kwargs)


@two_arguments_ops
async def pd_join(val1: Any, val2: Any) -> Any:
    return val1.join(val2)


async def pd_concat(objs: list, *args: Any, **kwargs: Any) -> Any:
    """TODO: How to input a list ?"""
    raise NotImplementedError



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sarus_data_spec.protobuf.path_pb2
import sarus_data_spec.protobuf.predicate_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class Type(google.protobuf.message.Message):
    """Types can be simple or composed

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    class PropertiesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text = ...
        value: typing.Text = ...
        def __init__(self,
            *,
            key : typing.Text = ...,
            value : typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    class Null(google.protobuf.message.Message):
        """Nested definitions

        Basic types are close to what can be found in:
        - Arrow: https://arrow.apache.org/docs/status.html#data-types
        - Tensorflow: https://www.tensorflow.org/api_docs/python/tf/dtypes/DType
        - Parquet: https://github.com/apache/parquet-format
        - Protobuf: https://developers.google.com/protocol-buffers/docs/proto3#scalar

        """
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        def __init__(self,
            ) -> None: ...

    class Unit(google.protobuf.message.Message):
        """A function returning Null type cannot return. It crashes or never returns. No value can actually have the Null type.
        This type is to indicate no value is possible.

        """
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        def __init__(self,
            ) -> None: ...

    class Boolean(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        def __init__(self,
            ) -> None: ...

    class Integer(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Base(_Base, metaclass=_BaseEnumTypeWrapper):
            pass
        class _Base:
            V = typing.NewType('V', builtins.int)
        class _BaseEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Base.V], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
            INT64 = Type.Integer.Base.V(0)
            INT32 = Type.Integer.Base.V(1)
            INT16 = Type.Integer.Base.V(2)
            INT8 = Type.Integer.Base.V(3)

        INT64 = Type.Integer.Base.V(0)
        INT32 = Type.Integer.Base.V(1)
        INT16 = Type.Integer.Base.V(2)
        INT8 = Type.Integer.Base.V(3)

        BASE_FIELD_NUMBER: builtins.int
        MIN_FIELD_NUMBER: builtins.int
        MAX_FIELD_NUMBER: builtins.int
        POSSIBLE_VALUES_FIELD_NUMBER: builtins.int
        base: global___Type.Integer.Base.V = ...
        min: builtins.int = ...
        max: builtins.int = ...
        @property
        def possible_values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(self,
            *,
            base : global___Type.Integer.Base.V = ...,
            min : builtins.int = ...,
            max : builtins.int = ...,
            possible_values : typing.Optional[typing.Iterable[builtins.int]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"base",b"base",u"max",b"max",u"min",b"min",u"possible_values",b"possible_values"]) -> None: ...

    class Enum(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Base(_Base, metaclass=_BaseEnumTypeWrapper):
            pass
        class _Base:
            V = typing.NewType('V', builtins.int)
        class _BaseEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Base.V], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
            INT64 = Type.Enum.Base.V(0)
            INT32 = Type.Enum.Base.V(1)
            INT16 = Type.Enum.Base.V(2)
            INT8 = Type.Enum.Base.V(3)

        INT64 = Type.Enum.Base.V(0)
        INT32 = Type.Enum.Base.V(1)
        INT16 = Type.Enum.Base.V(2)
        INT8 = Type.Enum.Base.V(3)

        class NameValue(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
            NAME_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            name: typing.Text = ...
            value: builtins.int = ...
            def __init__(self,
                *,
                name : typing.Text = ...,
                value : builtins.int = ...,
                ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal[u"name",b"name",u"value",b"value"]) -> None: ...

        BASE_FIELD_NUMBER: builtins.int
        ORDERED_FIELD_NUMBER: builtins.int
        NAME_VALUES_FIELD_NUMBER: builtins.int
        base: global___Type.Enum.Base.V = ...
        ordered: builtins.bool = ...
        @property
        def name_values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Type.Enum.NameValue]: ...
        def __init__(self,
            *,
            base : global___Type.Enum.Base.V = ...,
            ordered : builtins.bool = ...,
            name_values : typing.Optional[typing.Iterable[global___Type.Enum.NameValue]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"base",b"base",u"name_values",b"name_values",u"ordered",b"ordered"]) -> None: ...

    class Float(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Base(_Base, metaclass=_BaseEnumTypeWrapper):
            pass
        class _Base:
            V = typing.NewType('V', builtins.int)
        class _BaseEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Base.V], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
            FLOAT64 = Type.Float.Base.V(0)
            FLOAT32 = Type.Float.Base.V(1)
            FLOAT16 = Type.Float.Base.V(2)

        FLOAT64 = Type.Float.Base.V(0)
        FLOAT32 = Type.Float.Base.V(1)
        FLOAT16 = Type.Float.Base.V(2)

        BASE_FIELD_NUMBER: builtins.int
        MIN_FIELD_NUMBER: builtins.int
        MAX_FIELD_NUMBER: builtins.int
        POSSIBLE_VALUES_FIELD_NUMBER: builtins.int
        base: global___Type.Float.Base.V = ...
        min: builtins.float = ...
        max: builtins.float = ...
        @property
        def possible_values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
        def __init__(self,
            *,
            base : global___Type.Float.Base.V = ...,
            min : builtins.float = ...,
            max : builtins.float = ...,
            possible_values : typing.Optional[typing.Iterable[builtins.float]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"base",b"base",u"max",b"max",u"min",b"min",u"possible_values",b"possible_values"]) -> None: ...

    class Text(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        ENCODING_FIELD_NUMBER: builtins.int
        encoding: typing.Text = ...
        def __init__(self,
            *,
            encoding : typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"encoding",b"encoding"]) -> None: ...

    class Bytes(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        def __init__(self,
            ) -> None: ...

    class Struct(google.protobuf.message.Message):
        """Structured types"""
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Field(google.protobuf.message.Message):
            """A single field"""
            DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
            NAME_FIELD_NUMBER: builtins.int
            TYPE_FIELD_NUMBER: builtins.int
            name: typing.Text = ...
            @property
            def type(self) -> global___Type: ...
            def __init__(self,
                *,
                name : typing.Text = ...,
                type : typing.Optional[global___Type] = ...,
                ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal[u"name",b"name",u"type",b"type"]) -> None: ...

        FIELDS_FIELD_NUMBER: builtins.int
        @property
        def fields(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Type.Struct.Field]: ...
        def __init__(self,
            *,
            fields : typing.Optional[typing.Iterable[global___Type.Struct.Field]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"fields",b"fields"]) -> None: ...

    class Union(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Field(google.protobuf.message.Message):
            """A single field"""
            DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
            NAME_FIELD_NUMBER: builtins.int
            TYPE_FIELD_NUMBER: builtins.int
            name: typing.Text = ...
            @property
            def type(self) -> global___Type: ...
            def __init__(self,
                *,
                name : typing.Text = ...,
                type : typing.Optional[global___Type] = ...,
                ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal[u"name",b"name",u"type",b"type"]) -> None: ...

        FIELDS_FIELD_NUMBER: builtins.int
        @property
        def fields(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Type.Union.Field]:
            """A sum of type (tagged union)"""
            pass
        def __init__(self,
            *,
            fields : typing.Optional[typing.Iterable[global___Type.Union.Field]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"fields",b"fields"]) -> None: ...

    class Optional(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TYPE_FIELD_NUMBER: builtins.int
        @property
        def type(self) -> global___Type:
            """A value of type 'type' or null"""
            pass
        def __init__(self,
            *,
            type : typing.Optional[global___Type] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> None: ...

    class List(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TYPE_FIELD_NUMBER: builtins.int
        MAX_SIZE_FIELD_NUMBER: builtins.int
        @property
        def type(self) -> global___Type:
            """A repeated value of type 'type'"""
            pass
        max_size: builtins.int = ...
        def __init__(self,
            *,
            type : typing.Optional[global___Type] = ...,
            max_size : builtins.int = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"max_size",b"max_size",u"type",b"type"]) -> None: ...

    class Array(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TYPE_FIELD_NUMBER: builtins.int
        SHAPE_FIELD_NUMBER: builtins.int
        @property
        def type(self) -> global___Type:
            """A numpy-like n-dimensional array"""
            pass
        @property
        def shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        def __init__(self,
            *,
            type : typing.Optional[global___Type] = ...,
            shape : typing.Optional[typing.Iterable[builtins.int]] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"shape",b"shape",u"type",b"type"]) -> None: ...

    class Datetime(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Base(_Base, metaclass=_BaseEnumTypeWrapper):
            pass
        class _Base:
            V = typing.NewType('V', builtins.int)
        class _BaseEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Base.V], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
            INT64_NS = Type.Datetime.Base.V(0)
            INT64_MS = Type.Datetime.Base.V(1)
            STRING = Type.Datetime.Base.V(2)

        INT64_NS = Type.Datetime.Base.V(0)
        INT64_MS = Type.Datetime.Base.V(1)
        STRING = Type.Datetime.Base.V(2)

        FORMAT_FIELD_NUMBER: builtins.int
        MIN_FIELD_NUMBER: builtins.int
        MAX_FIELD_NUMBER: builtins.int
        POSSIBLE_VALUES_FIELD_NUMBER: builtins.int
        BASE_FIELD_NUMBER: builtins.int
        format: typing.Text = ...
        """A datetime type"""

        min: typing.Text = ...
        max: typing.Text = ...
        @property
        def possible_values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
        base: global___Type.Datetime.Base.V = ...
        def __init__(self,
            *,
            format : typing.Text = ...,
            min : typing.Text = ...,
            max : typing.Text = ...,
            possible_values : typing.Optional[typing.Iterable[typing.Text]] = ...,
            base : global___Type.Datetime.Base.V = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"base",b"base",u"format",b"format",u"max",b"max",u"min",b"min",u"possible_values",b"possible_values"]) -> None: ...

    class Constrained(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TYPE_FIELD_NUMBER: builtins.int
        CONSTRAINT_FIELD_NUMBER: builtins.int
        @property
        def type(self) -> global___Type:
            """A type for simple relations"""
            pass
        @property
        def constraint(self) -> sarus_data_spec.protobuf.predicate_pb2.Predicate: ...
        def __init__(self,
            *,
            type : typing.Optional[global___Type] = ...,
            constraint : typing.Optional[sarus_data_spec.protobuf.predicate_pb2.Predicate] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"constraint",b"constraint",u"type",b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"constraint",b"constraint",u"type",b"type"]) -> None: ...

    class Hypothesis(google.protobuf.message.Message):
        """A constructor to express an uncertain type"""
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Scored(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
            TYPE_FIELD_NUMBER: builtins.int
            SCORE_FIELD_NUMBER: builtins.int
            @property
            def type(self) -> global___Type:
                """A type with a score (log-probability)"""
                pass
            score: builtins.float = ...
            def __init__(self,
                *,
                type : typing.Optional[global___Type] = ...,
                score : builtins.float = ...,
                ) -> None: ...
            def HasField(self, field_name: typing_extensions.Literal[u"type",b"type"]) -> builtins.bool: ...
            def ClearField(self, field_name: typing_extensions.Literal[u"score",b"score",u"type",b"type"]) -> None: ...

        TYPES_FIELD_NUMBER: builtins.int
        @property
        def types(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Type.Hypothesis.Scored]:
            """A type among those with a score"""
            pass
        def __init__(self,
            *,
            types : typing.Optional[typing.Iterable[global___Type.Hypothesis.Scored]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"types",b"types"]) -> None: ...

    class Id(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class Base(_Base, metaclass=_BaseEnumTypeWrapper):
            pass
        class _Base:
            V = typing.NewType('V', builtins.int)
        class _BaseEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Base.V], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
            INT64 = Type.Id.Base.V(0)
            INT32 = Type.Id.Base.V(1)
            INT16 = Type.Id.Base.V(2)
            INT8 = Type.Id.Base.V(3)
            STRING = Type.Id.Base.V(4)
            BYTES = Type.Id.Base.V(5)

        INT64 = Type.Id.Base.V(0)
        INT32 = Type.Id.Base.V(1)
        INT16 = Type.Id.Base.V(2)
        INT8 = Type.Id.Base.V(3)
        STRING = Type.Id.Base.V(4)
        BYTES = Type.Id.Base.V(5)

        BASE_FIELD_NUMBER: builtins.int
        UNIQUE_FIELD_NUMBER: builtins.int
        REFERENCE_FIELD_NUMBER: builtins.int
        base: global___Type.Id.Base.V = ...
        unique: builtins.bool = ...
        @property
        def reference(self) -> sarus_data_spec.protobuf.path_pb2.Path: ...
        def __init__(self,
            *,
            base : global___Type.Id.Base.V = ...,
            unique : builtins.bool = ...,
            reference : typing.Optional[sarus_data_spec.protobuf.path_pb2.Path] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"reference",b"reference"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"base",b"base",u"reference",b"reference",u"unique",b"unique"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    NULL_FIELD_NUMBER: builtins.int
    UNIT_FIELD_NUMBER: builtins.int
    BOOLEAN_FIELD_NUMBER: builtins.int
    INTEGER_FIELD_NUMBER: builtins.int
    ENUM_FIELD_NUMBER: builtins.int
    FLOAT_FIELD_NUMBER: builtins.int
    TEXT_FIELD_NUMBER: builtins.int
    BYTES_FIELD_NUMBER: builtins.int
    STRUCT_FIELD_NUMBER: builtins.int
    UNION_FIELD_NUMBER: builtins.int
    OPTIONAL_FIELD_NUMBER: builtins.int
    LIST_FIELD_NUMBER: builtins.int
    ARRAY_FIELD_NUMBER: builtins.int
    DATETIME_FIELD_NUMBER: builtins.int
    CONSTRAINED_FIELD_NUMBER: builtins.int
    HYPOTHESIS_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    name: typing.Text = ...
    """Type definition"""

    @property
    def null(self) -> global___Type.Null: ...
    @property
    def unit(self) -> global___Type.Unit: ...
    @property
    def boolean(self) -> global___Type.Boolean: ...
    @property
    def integer(self) -> global___Type.Integer: ...
    @property
    def enum(self) -> global___Type.Enum: ...
    @property
    def float(self) -> global___Type.Float: ...
    @property
    def text(self) -> global___Type.Text: ...
    @property
    def bytes(self) -> global___Type.Bytes: ...
    @property
    def struct(self) -> global___Type.Struct: ...
    @property
    def union(self) -> global___Type.Union: ...
    @property
    def optional(self) -> global___Type.Optional: ...
    @property
    def list(self) -> global___Type.List: ...
    @property
    def array(self) -> global___Type.Array: ...
    @property
    def datetime(self) -> global___Type.Datetime: ...
    @property
    def constrained(self) -> global___Type.Constrained: ...
    @property
    def hypothesis(self) -> global___Type.Hypothesis: ...
    @property
    def id(self) -> global___Type.Id: ...
    @property
    def properties(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]:
        """Other properties"""
        pass
    def __init__(self,
        *,
        name : typing.Text = ...,
        null : typing.Optional[global___Type.Null] = ...,
        unit : typing.Optional[global___Type.Unit] = ...,
        boolean : typing.Optional[global___Type.Boolean] = ...,
        integer : typing.Optional[global___Type.Integer] = ...,
        enum : typing.Optional[global___Type.Enum] = ...,
        float : typing.Optional[global___Type.Float] = ...,
        text : typing.Optional[global___Type.Text] = ...,
        bytes : typing.Optional[global___Type.Bytes] = ...,
        struct : typing.Optional[global___Type.Struct] = ...,
        union : typing.Optional[global___Type.Union] = ...,
        optional : typing.Optional[global___Type.Optional] = ...,
        list : typing.Optional[global___Type.List] = ...,
        array : typing.Optional[global___Type.Array] = ...,
        datetime : typing.Optional[global___Type.Datetime] = ...,
        constrained : typing.Optional[global___Type.Constrained] = ...,
        hypothesis : typing.Optional[global___Type.Hypothesis] = ...,
        id : typing.Optional[global___Type.Id] = ...,
        properties : typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"array",b"array",u"boolean",b"boolean",u"bytes",b"bytes",u"constrained",b"constrained",u"datetime",b"datetime",u"enum",b"enum",u"float",b"float",u"hypothesis",b"hypothesis",u"id",b"id",u"integer",b"integer",u"list",b"list",u"null",b"null",u"optional",b"optional",u"struct",b"struct",u"text",b"text",u"type",b"type",u"union",b"union",u"unit",b"unit"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"array",b"array",u"boolean",b"boolean",u"bytes",b"bytes",u"constrained",b"constrained",u"datetime",b"datetime",u"enum",b"enum",u"float",b"float",u"hypothesis",b"hypothesis",u"id",b"id",u"integer",b"integer",u"list",b"list",u"name",b"name",u"null",b"null",u"optional",b"optional",u"properties",b"properties",u"struct",b"struct",u"text",b"text",u"type",b"type",u"union",b"union",u"unit",b"unit"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal[u"type",b"type"]) -> typing.Optional[typing_extensions.Literal["null","unit","boolean","integer","enum","float","text","bytes","struct","union","optional","list","array","datetime","constrained","hypothesis","id"]]: ...
global___Type = Type



import pyarrow as pa

import sarus_data_spec.typing as st


def convert_record_batch(
    record_batch: pa.RecordBatch, _type: st.Type
) -> pa.Array:

    if (
        _type.protobuf().WhichOneof('type')  # type:ignore
        not in ['struct', 'union']
        == 1
    ):
        return record_batch.column(0)
    if _type.protobuf().HasField('struct'):
        names = list(_type.children().keys())
    elif _type.protobuf().HasField('union'):
        names = list(_type.children().keys())
        names.append('field_selected')
    else:
        raise TypeError('got {type} but many columns in the batch array')
    return pa.StructArray.from_arrays(record_batch.columns, names=names)



from typing import Any

from .utils import one_parent_ops


@one_parent_ops
async def np_mean(val: Any, *args: Any, **kwargs: Any) -> Any:
    return val.mean(*args, **kwargs)


@one_parent_ops
async def np_std(val: Any, *args: Any, **kwargs: Any) -> Any:
    return val.std(*args, **kwargs)



from __future__ import annotations

from typing import List, Optional, Type, cast

from sarus_data_spec.base import Referring
import sarus_data_spec.protobuf as sp
import sarus_data_spec.typing as st


class VariantConstraint(Referring[sp.VariantConstraint]):
    def __init__(self, protobuf: sp.VariantConstraint) -> None:
        self._referred = {protobuf.dataspec}
        super().__init__(protobuf=protobuf)

    def prototype(self) -> Type[sp.VariantConstraint]:
        """Return the type of the underlying protobuf."""
        return sp.VariantConstraint

    def constraint_kind(self) -> st.ConstraintKind:
        return st.ConstraintKind(self.protobuf().constraint_kind)

    def required_context(self) -> List[str]:
        return list(self.protobuf().required_context)

    def epsilon(self) -> Optional[float]:
        str_epsilon = self.protobuf().properties.get("epsilon", None)
        if str_epsilon:
            return float(str_epsilon)
        else:
            return None

    def dataspec(self) -> st.DataSpec:
        return cast(
            st.DataSpec, self.storage().referrable(self.protobuf().dataspec)
        )


def variant_constraint(
    constraint_kind: st.ConstraintKind,
    dataspec: st.DataSpec,
    required_context: List[str] = [],
    epsilon: Optional[float] = None,
) -> VariantConstraint:
    if epsilon:
        properties = {"epsilon": str(epsilon)}
    else:
        properties = dict()

    return VariantConstraint(
        sp.VariantConstraint(
            dataspec=dataspec.uuid(),
            constraint_kind=sp.ConstraintKind.Value(constraint_kind.name),
            properties=properties,
            required_context=required_context,
        )
    )


def dp_constraint(
    required_context: List[str],
    epsilon: float,
    dataspec: str,
) -> VariantConstraint:
    return VariantConstraint(
        sp.VariantConstraint(
            constraint_kind=sp.ConstraintKind.DP,
            required_context=required_context,
            properties={"epsilon": str(epsilon)},
            dataspec=dataspec,
        )
    )


def pep_constraint(
    required_context: List[str],
    epsilon: float,
    dataspec: str,
) -> VariantConstraint:
    return VariantConstraint(
        sp.VariantConstraint(
            constraint_kind=sp.ConstraintKind.PEP,
            required_context=required_context,
            properties={"epsilon": str(epsilon)},
            dataspec=dataspec,
        )
    )


def syn_constraint(
    required_context: List[str],
    dataspec: str,
) -> VariantConstraint:
    return VariantConstraint(
        sp.VariantConstraint(
            constraint_kind=sp.ConstraintKind.SYNTHETIC,
            required_context=required_context,
            dataspec=dataspec,
        )
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/predicate.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/predicate.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n(sarus_data_spec/protobuf/predicate.proto\x12\x0fsarus_data_spec\"\xab\x04\n\tPredicate\x12\x33\n\x06simple\x18\x01 \x01(\x0b\x32!.sarus_data_spec.Predicate.SimpleH\x00\x12\x31\n\x05inter\x18\x02 \x01(\x0b\x32 .sarus_data_spec.Predicate.InterH\x00\x12\x31\n\x05union\x18\x03 \x01(\x0b\x32 .sarus_data_spec.Predicate.UnionH\x00\x12/\n\x04\x63omp\x18\x04 \x01(\x0b\x32\x1f.sarus_data_spec.Predicate.CompH\x00\x12>\n\nproperties\x18\x05 \x03(\x0b\x32*.sarus_data_spec.Predicate.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a)\n\x06Simple\x12\x10\n\x08operator\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x1a\x37\n\x05Inter\x12.\n\npredicates\x18\x01 \x03(\x0b\x32\x1a.sarus_data_spec.Predicate\x1a\x37\n\x05Union\x12.\n\npredicates\x18\x01 \x03(\x0b\x32\x1a.sarus_data_spec.Predicate\x1a\x35\n\x04\x43omp\x12-\n\tpredicate\x18\x01 \x01(\x0b\x32\x1a.sarus_data_spec.PredicateB\x0b\n\tpredicateb\x06proto3'
)




_PREDICATE_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Predicate.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Predicate.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Predicate.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=343,
  serialized_end=392,
)

_PREDICATE_SIMPLE = _descriptor.Descriptor(
  name='Simple',
  full_name='sarus_data_spec.Predicate.Simple',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operator', full_name='sarus_data_spec.Predicate.Simple.operator', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Predicate.Simple.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=394,
  serialized_end=435,
)

_PREDICATE_INTER = _descriptor.Descriptor(
  name='Inter',
  full_name='sarus_data_spec.Predicate.Inter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='predicates', full_name='sarus_data_spec.Predicate.Inter.predicates', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=437,
  serialized_end=492,
)

_PREDICATE_UNION = _descriptor.Descriptor(
  name='Union',
  full_name='sarus_data_spec.Predicate.Union',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='predicates', full_name='sarus_data_spec.Predicate.Union.predicates', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=494,
  serialized_end=549,
)

_PREDICATE_COMP = _descriptor.Descriptor(
  name='Comp',
  full_name='sarus_data_spec.Predicate.Comp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='predicate', full_name='sarus_data_spec.Predicate.Comp.predicate', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=551,
  serialized_end=604,
)

_PREDICATE = _descriptor.Descriptor(
  name='Predicate',
  full_name='sarus_data_spec.Predicate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='simple', full_name='sarus_data_spec.Predicate.simple', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inter', full_name='sarus_data_spec.Predicate.inter', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='union', full_name='sarus_data_spec.Predicate.union', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='comp', full_name='sarus_data_spec.Predicate.comp', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Predicate.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_PREDICATE_PROPERTIESENTRY, _PREDICATE_SIMPLE, _PREDICATE_INTER, _PREDICATE_UNION, _PREDICATE_COMP, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='predicate', full_name='sarus_data_spec.Predicate.predicate',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=62,
  serialized_end=617,
)

_PREDICATE_PROPERTIESENTRY.containing_type = _PREDICATE
_PREDICATE_SIMPLE.containing_type = _PREDICATE
_PREDICATE_INTER.fields_by_name['predicates'].message_type = _PREDICATE
_PREDICATE_INTER.containing_type = _PREDICATE
_PREDICATE_UNION.fields_by_name['predicates'].message_type = _PREDICATE
_PREDICATE_UNION.containing_type = _PREDICATE
_PREDICATE_COMP.fields_by_name['predicate'].message_type = _PREDICATE
_PREDICATE_COMP.containing_type = _PREDICATE
_PREDICATE.fields_by_name['simple'].message_type = _PREDICATE_SIMPLE
_PREDICATE.fields_by_name['inter'].message_type = _PREDICATE_INTER
_PREDICATE.fields_by_name['union'].message_type = _PREDICATE_UNION
_PREDICATE.fields_by_name['comp'].message_type = _PREDICATE_COMP
_PREDICATE.fields_by_name['properties'].message_type = _PREDICATE_PROPERTIESENTRY
_PREDICATE.oneofs_by_name['predicate'].fields.append(
  _PREDICATE.fields_by_name['simple'])
_PREDICATE.fields_by_name['simple'].containing_oneof = _PREDICATE.oneofs_by_name['predicate']
_PREDICATE.oneofs_by_name['predicate'].fields.append(
  _PREDICATE.fields_by_name['inter'])
_PREDICATE.fields_by_name['inter'].containing_oneof = _PREDICATE.oneofs_by_name['predicate']
_PREDICATE.oneofs_by_name['predicate'].fields.append(
  _PREDICATE.fields_by_name['union'])
_PREDICATE.fields_by_name['union'].containing_oneof = _PREDICATE.oneofs_by_name['predicate']
_PREDICATE.oneofs_by_name['predicate'].fields.append(
  _PREDICATE.fields_by_name['comp'])
_PREDICATE.fields_by_name['comp'].containing_oneof = _PREDICATE.oneofs_by_name['predicate']
DESCRIPTOR.message_types_by_name['Predicate'] = _PREDICATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Predicate = _reflection.GeneratedProtocolMessageType('Predicate', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _PREDICATE_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.predicate_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Predicate.PropertiesEntry)
    })
  ,

  'Simple' : _reflection.GeneratedProtocolMessageType('Simple', (_message.Message,), {
    'DESCRIPTOR' : _PREDICATE_SIMPLE,
    '__module__' : 'sarus_data_spec.protobuf.predicate_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Predicate.Simple)
    })
  ,

  'Inter' : _reflection.GeneratedProtocolMessageType('Inter', (_message.Message,), {
    'DESCRIPTOR' : _PREDICATE_INTER,
    '__module__' : 'sarus_data_spec.protobuf.predicate_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Predicate.Inter)
    })
  ,

  'Union' : _reflection.GeneratedProtocolMessageType('Union', (_message.Message,), {
    'DESCRIPTOR' : _PREDICATE_UNION,
    '__module__' : 'sarus_data_spec.protobuf.predicate_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Predicate.Union)
    })
  ,

  'Comp' : _reflection.GeneratedProtocolMessageType('Comp', (_message.Message,), {
    'DESCRIPTOR' : _PREDICATE_COMP,
    '__module__' : 'sarus_data_spec.protobuf.predicate_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Predicate.Comp)
    })
  ,
  'DESCRIPTOR' : _PREDICATE,
  '__module__' : 'sarus_data_spec.protobuf.predicate_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.Predicate)
  })
_sym_db.RegisterMessage(Predicate)
_sym_db.RegisterMessage(Predicate.PropertiesEntry)
_sym_db.RegisterMessage(Predicate.Simple)
_sym_db.RegisterMessage(Predicate.Inter)
_sym_db.RegisterMessage(Predicate.Union)
_sym_db.RegisterMessage(Predicate.Comp)


_PREDICATE_PROPERTIESENTRY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/constraint.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/constraint.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n)sarus_data_spec/protobuf/constraint.proto\x12\x0fsarus_data_spec\"\x82\x02\n\x11VariantConstraint\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61taspec\x18\x02 \x01(\t\x12\x38\n\x0f\x63onstraint_kind\x18\x03 \x01(\x0e\x32\x1f.sarus_data_spec.ConstraintKind\x12\x18\n\x10required_context\x18\x04 \x03(\t\x12\x46\n\nproperties\x18\x05 \x03(\x0b\x32\x32.sarus_data_spec.VariantConstraint.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01*<\n\x0e\x43onstraintKind\x12\r\n\tSYNTHETIC\x10\x00\x12\x06\n\x02\x44P\x10\x01\x12\n\n\x06PUBLIC\x10\x02\x12\x07\n\x03PEP\x10\x03\x62\x06proto3'
)

_CONSTRAINTKIND = _descriptor.EnumDescriptor(
  name='ConstraintKind',
  full_name='sarus_data_spec.ConstraintKind',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SYNTHETIC', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DP', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PUBLIC', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PEP', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=323,
  serialized_end=383,
)
_sym_db.RegisterEnumDescriptor(_CONSTRAINTKIND)

ConstraintKind = enum_type_wrapper.EnumTypeWrapper(_CONSTRAINTKIND)
SYNTHETIC = 0
DP = 1
PUBLIC = 2
PEP = 3



_VARIANTCONSTRAINT_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.VariantConstraint.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.VariantConstraint.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.VariantConstraint.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=272,
  serialized_end=321,
)

_VARIANTCONSTRAINT = _descriptor.Descriptor(
  name='VariantConstraint',
  full_name='sarus_data_spec.VariantConstraint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='sarus_data_spec.VariantConstraint.uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dataspec', full_name='sarus_data_spec.VariantConstraint.dataspec', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='constraint_kind', full_name='sarus_data_spec.VariantConstraint.constraint_kind', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='required_context', full_name='sarus_data_spec.VariantConstraint.required_context', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.VariantConstraint.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_VARIANTCONSTRAINT_PROPERTIESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=63,
  serialized_end=321,
)

_VARIANTCONSTRAINT_PROPERTIESENTRY.containing_type = _VARIANTCONSTRAINT
_VARIANTCONSTRAINT.fields_by_name['constraint_kind'].enum_type = _CONSTRAINTKIND
_VARIANTCONSTRAINT.fields_by_name['properties'].message_type = _VARIANTCONSTRAINT_PROPERTIESENTRY
DESCRIPTOR.message_types_by_name['VariantConstraint'] = _VARIANTCONSTRAINT
DESCRIPTOR.enum_types_by_name['ConstraintKind'] = _CONSTRAINTKIND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VariantConstraint = _reflection.GeneratedProtocolMessageType('VariantConstraint', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _VARIANTCONSTRAINT_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.constraint_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.VariantConstraint.PropertiesEntry)
    })
  ,
  'DESCRIPTOR' : _VARIANTCONSTRAINT,
  '__module__' : 'sarus_data_spec.protobuf.constraint_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.VariantConstraint)
  })
_sym_db.RegisterMessage(VariantConstraint)
_sym_db.RegisterMessage(VariantConstraint.PropertiesEntry)


_VARIANTCONSTRAINT_PROPERTIESENTRY._options = None
# @@protoc_insertion_point(module_scope)
